# -*- coding: utf-8 -*-

import logging
import os
import re
from collections import Mapping, UserDict
from logging import Logger
from pathlib import Path
from typing import Callable, Dict, List, Tuple

import yaml

LOGGER = logging.getLogger(__name__)


class AliasDict(UserDict):
    """
      Alias Dictionary
    """

    logger = LOGGER.getChild('AliasDict')  # type: logging.Logger

    @classmethod
    def load_from_yaml(cls, path: Path, **kargs) -> 'AliasDict':
        """
          load from yaml
        """

        assert path.exists()

        with path.open('r', encoding='utf-8') as f:
            return cls(yaml.load(f), **kargs)

    def __init__(self, config: dict,
                 replace_pattern: str = '\${(.*)}',
                 suffix_handler={
                     'dir': lambda val: os.path.expanduser(val)
                 }):

        self._replace_pattern = re.compile(replace_pattern)
        self._cache = {}
        self._suffix_handler = suffix_handler

        self.data = config

    def _replacer(self, match):
        return self.get(match.group()[2:-1])

    def get(self, key: str, default=None):
        if key in self._cache:
            return self._cache.get(key)

        keys = key.split('.')
        last = keys[-1]

        try:
            ret = self.__get(self.data, keys)
            if isinstance(ret, str):
                ret = self._replace_pattern.sub(self._replacer, ret)

            ret = self._suffix_handler.get(last, lambda x: x)(ret)

            self._cache.setdefault(key, ret)
            return ret
        except KeyError:
            self.logger.error('Not found spec key:%s', keys)
            return None

    def __get(self, config, keys):

        if len(keys) == 1:
            return config[keys[0]]
        else:
            return self.__get(config[keys[0]], keys[1:])

    def update(self, u):
        if isinstance(u, AliasDict):
            u = u.data

        self.data = AliasDict.deep_update(self.data, u)

    @classmethod
    def deep_update(cls, d, u):
        for k, v in u.items():
            if isinstance(v, Mapping):
                r = cls.deep_update(d.get(k, {}), v)
                d[k] = r
            else:
                d[k] = u[k]

        return d


def split_qualified_name(qualified_name: str):

    qualified_token = qualified_name.split('.')  # type: str
    _package = '.'.join(qualified_token[:-1])
    _callable = qualified_token[-1]

    return _package, _callable



# -*- coding: utf-8 -*-

import importlib
import logging
import os
import sys
from argparse import ArgumentParser
from pathlib import Path
from typing import Callable

import yaml

from invoker.utils import split_qualified_name

LOGGER = logging.getLogger(__name__)


def define_args_parser() -> ArgumentParser:

    parser = ArgumentParser(description="invoke specify function with DI")

    parser.add_argument('log_config', help='specify logging.yaml path', type=Path)
    parser.add_argument('invoke_options', help='specify invoke.yaml path')
    parser.add_argument('--config', help='specify configuration.yaml path', type=Path, action='append')
    parser.add_argument('--invoke_context_class', help='specify InvokerContext subclass.', type=str, default='invoker.InvokerContext')

    return parser


def execute():

    parser = define_args_parser()  # type* ArgumentParser
    args = parser.parse_args()

    _package, _callable = split_qualified_name(args.invoke_context_class)
    _InvokerClass = getattr(importlib.import_module(_package), _callable)  # type: Callable

    with Path(args.invoke_options).open('r', encoding='utf-8') as f:
        invoke_options = yaml.load(f)

    invoker = _InvokerClass(args.config, args.log_config)
    ret = invoker.invoke(invoke_options)

    LOGGER.info('invoke ret: %s', ret)

    sys.exit(0)


if __name__ == '__main__':
    execute()



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="invoker",
    version="1.0.0",
    description='Invoker utility for DI(Injector) and context configuration.',
    packages=find_packages(),
    install_requires=["injector==0.14.1", "pyyaml>=4.2b1"],
    extras_require={
        "develop": ["autopep8", "pep8"]
    },
    entry_points={
        "console_scripts": [
            "invoker = invoker.cmd:execute"
        ],
    }
)
